package dev.pfaff.altargui;

import dev.pfaff.altargui.debug.FakeConstantProfiler;
import dev.pfaff.altargui.debug.Profiler;
import dev.pfaff.altargui.host.Host;
import dev.pfaff.altargui.internal.InternalGlobals;
import dev.pfaff.glass.opengl.GL;
import dev.pfaff.log4truth.NamedLogger;
import org.jspecify.annotations.Nullable;

import java.lang.invoke.MethodHandles;

import static java.util.Objects.requireNonNull;

/**
 * Some resources are held here as program globals for performance or complexity reasons.
 */
public final class GuiGlobals {
	public static final NamedLogger LOGGER = new NamedLogger("AltarGUI");

	public static final FakeConstantProfiler PROFILER = new FakeConstantProfiler(Profiler.NOP);

	public static final Host HOST;
	public static final GL GL;

	static {
		//noinspection DataFlowIssue
		HOST = Init.host;
		if (HOST == null) {
			throw new ExceptionInInitializerError(new NullPointerException("host is uninitialized"));
		}

		GL = HOST.gl();
		//noinspection ConstantValue
		if (GL == null) {
			throw new ExceptionInInitializerError(new NullPointerException("host.gl() is null"));
		}

		try {
			MethodHandles.lookup().ensureInitialized(InternalGlobals.class);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	public static Profiler profiler() {
		return PROFILER.get();
	}

	public static final class Init {
		static @Nullable Host host;

		public static void initialize(Host hostIn) {
			if (host != null) throw new IllegalStateException("host is already initialized");
			host = requireNonNull(hostIn);

			try {
				MethodHandles.lookup().ensureInitialized(GuiGlobals.class);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		}
	}
}
