package dev.pfaff.altargui.render.objects;

import dev.pfaff.altargui.CanvasImpl;
import dev.pfaff.altargui.render.LayeredRenderBuffer;
import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.render.ViaIntermediateBuffer;
import dev.pfaff.altargui.texture.TextureHandle;
import dev.pfaff.altargui.types.Tex2d;

import java.lang.foreign.MemoryLayout;
import java.lang.foreign.StructLayout;

import static dev.pfaff.altargui.render.BufferUtil.*;
import static dev.pfaff.altargui.shader.Programs.TEXTURED_RECT_INSTANCE_SPEC;
import static dev.pfaff.altargui.shader.Programs.texturedRectRenderLayer;
import static java.util.FormatProcessor.FMT;

public final class TexturedRectRenderObject extends IRectRenderObject {
	public static final StructLayout LAYOUT = TEXTURED_RECT_INSTANCE_SPEC.layout();

	private char uMin, vMin, uMaxMinusOne, vMaxMinusOne;

	/**
	 * Ignores texture id.
	 */
	public void updateTex(TextureHandle handle, Tex2d tex) {
		float u1 = tex.u1();
		float v1 = tex.v1();
		float u2 = tex.u2();
		float v2 = tex.v2();
		this.uMin = (char) handle.texToAtlasX(u1);
		this.vMin = (char) handle.texToAtlasY(v1);
		this.uMaxMinusOne = (char) (handle.texToAtlasX(u2) - 1);
		this.vMaxMinusOne = (char) (handle.texToAtlasY(v2) - 1);
	}

	@Override
	public MemoryLayout instanceLayout() {
		return LAYOUT;
	}

	@Override
	public int size() {
		return super.size();
	}

	@Override
	public void uploadTo(Sink sink) {
		sink.checkRemaining(LAYOUT.byteSize());
		emitVec2(sink, getX(), getY());
		emitVec2(sink, getWidth(), getHeight());
//		emitVec2AsNormUShort(sink, u, v);
//		emitVec2AsNormUShort(sink, uExtent, vExtent);
		emitCVec2(sink, this.uMin, this.vMin);
		emitCVec2(sink, this.uMaxMinusOne, this.vMaxMinusOne);
		emitShaderColor(sink, getColor());
	}

	@Override
	public void uSpecUploadAtUnchecked(ViaIntermediateBuffer buffer, long offset) {
		super.uSpecUploadAtUnchecked(buffer, offset);
	}

	@Override
	public void uSpecUploadCanvas(CanvasImpl canvas) {
		super.uSpecUploadCanvas(canvas);
	}

	@Override
	public void uSpecUploadLayered(LayeredRenderBuffer buffer) {
		super.uSpecUploadLayered(buffer);
	}

	@Override
	public int program() {
		return texturedRectRenderLayer();
	}

	@Override
	public void close() {
	}

	@Override
	public String toString() {
		return FMT."""
			TexturedRectRenderObject[x=\{getX()}, y=\{getY()}, width=\{getWidth()}, height=\{getHeight()}, \
			uMin=\{uMin}, vMin=\{vMin}, uMaxMinusOne=\{uMaxMinusOne}, vMaxMinusOne=\{vMaxMinusOne}, \
			color=%08x\{getColor().rgba()}]""";
	}
}
