package dev.pfaff.altargui.util;

import dev.pfaff.altargui.async.TaskExecutor;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MutableCallSite;

import static java.lang.invoke.MethodType.methodType;

public final class GLThread {
	private static final MutableCallSite EXECUTOR = new MutableCallSite(methodType(TaskExecutor.class));
	private static final MutableCallSite ASYNC_EXECUTOR = new MutableCallSite(methodType(TaskExecutor.class));
	private static final MethodHandle MH_identity_TaskExecutor = MethodHandles.identity(TaskExecutor.class);

	public static void setExecutor(TaskExecutor executor) {
		EXECUTOR.setTarget(MH_identity_TaskExecutor.bindTo(executor));
	}

	public static void setAsyncExecutor(TaskExecutor executor) {
		ASYNC_EXECUTOR.setTarget(MH_identity_TaskExecutor.bindTo(executor));
	}

	/**
	 * If the current thread is the GL thread, the {@code task} <strong>may</strong> be executed immediately.
	 */
	public static TaskExecutor executor() {
		try {
			return (TaskExecutor) EXECUTOR.getTarget().invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * If the current thread is the GL thread, the {@code task} will be queued for later execution. Do not attempt to
	 * call any of the {@link TaskExecutor#execute execute*} methods on this executor.
	 */
	public static TaskExecutor asyncExecutor() {
		try {
			return (TaskExecutor) ASYNC_EXECUTOR.getTarget().invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
